@@ -6,6 +6,9 @@ import numpy as np
 import matplotlib
 import matplotlib.pyplot as plt
 from matplotlib import cm #EAW 2020/01/07
+import platform
+if platform.system() == 'posix':
+    matplotlib.use('Agg') #added by A. Savara June 29th, 2021.
 import copy
 
 class plotting_functions_class():
@@ -107,7 +110,7 @@ class plotting_functions_class():
     def rate_tot_plot(self):
         return
 
-def sampledParameterHistogramMaker(parameterSamples, parameterName,parameterNamesAndMathTypeExpressionsDict, sampledParameterFiguresDictionary, sampledParameterAxesDictionary):
+def sampledParameterHistogramMaker(parameterSamples, parameterName, parameterNamesAndMathTypeExpressionsDict, sampledParameterFiguresDictionary, sampledParameterAxesDictionary, directory='', parameterInitialValue=None, parameterMAPValue=None, parameterMuAPValue=None):
         parameterIndex = list(parameterNamesAndMathTypeExpressionsDict).index(parameterName)
         sampledParameterFiguresDictionary[parameterName], sampledParameterAxesDictionary[parameterName] = plt.subplots()   #making plt objects    
         sampledParameterAxesDictionary[parameterName].hist(parameterSamples[:,parameterIndex]) #filling the object with data
@@ -115,11 +118,17 @@ def sampledParameterHistogramMaker(parameterSamples, parameterName,parameterName
         axis_font = {'size':'16'} #TODO: Make this size a setting that can be changed. #Code was made following answer by "binaryfunt" here https://stackoverflow.com/questions/3899980/how-to-change-the-font-size-on-a-matplotlib-plot
         sampledParameterAxesDictionary[parameterName].set_ylabel('frequency', **axis_font)
         sampledParameterAxesDictionary[parameterName].set_xlabel(parameterNamesAndMathTypeExpressionsDict[parameterName], **axis_font)
+        if parameterInitialValue!=None:
+            sampledParameterAxesDictionary[parameterName].axvline(x=parameterInitialValue, color='#00A5DF', linestyle='--')
+        if parameterMAPValue!=None:
+            sampledParameterAxesDictionary[parameterName].axvline(x=parameterMAPValue, color='r', linestyle='--')
+        if parameterMuAPValue!=None:
+            sampledParameterAxesDictionary[parameterName].axvline(x=parameterMuAPValue, color='k', linestyle='--')
 
         sampledParameterAxesDictionary[parameterName].tick_params(axis='x', labelsize=16) #TODO: make these labels sizes a setting that can be changed.
         sampledParameterAxesDictionary[parameterName].tick_params(axis='y', labelsize=16)
         sampledParameterFiguresDictionary[parameterName].tight_layout()
-        sampledParameterFiguresDictionary[parameterName].savefig('Histogram_sampling_'+parameterName+'.png', dpi=220)
+        sampledParameterFiguresDictionary[parameterName].savefig(directory+'Histogram_sampling_'+str(parameterIndex)+'_'+parameterName+'.png', dpi=220)
 
         
         
@@ -132,14 +141,18 @@ def sampledParameterHistogramMaker(parameterSamples, parameterName,parameterName
         # fig2.savefig('Ea2.png', dpi=220)
 
     #Make histograms for each parameter. Need to make some dictionaries where relevant objects will be stored.
-def makeHistogramsForEachParameter(parameterSamples,parameterNamesAndMathTypeExpressionsDict):
+def makeHistogramsForEachParameter(parameterSamples,parameterNamesAndMathTypeExpressionsDict, directory='', parameterInitialValue=None, parameterMAPValue=None, parameterMuAPValue=None):
     sampledParameterFiguresDictionary = copy.deepcopy(parameterNamesAndMathTypeExpressionsDict) #This must be a deep copy to perserve original.
     sampledParameterAxesDictionary = copy.deepcopy(parameterNamesAndMathTypeExpressionsDict) #This must be a deep copy to preserve original.
-    for key in parameterNamesAndMathTypeExpressionsDict:
+    #The below code was originally added by Eric Walker, then modified by Troy Gustke to add in the initialValue, MAPvalue, and mu_apvALUE in June 2021, and merged in by A. Savara.    
+    for key, iv, mp, mup in zip(parameterNamesAndMathTypeExpressionsDict,parameterInitialValue,parameterMAPValue,parameterMuAPValue):
         parameterName = key
-        sampledParameterHistogramMaker(parameterSamples, parameterName,parameterNamesAndMathTypeExpressionsDict, sampledParameterFiguresDictionary, sampledParameterAxesDictionary)        
+        initialValue = iv
+        MAPValue = mp
+        Mu_APValue = mup
+        sampledParameterHistogramMaker(parameterSamples, parameterName, parameterNamesAndMathTypeExpressionsDict, sampledParameterFiguresDictionary, sampledParameterAxesDictionary, directory=directory, parameterInitialValue=initialValue, parameterMAPValue=MAPValue, parameterMuAPValue=Mu_APValue)        
 
-def createSimulatedResponsesPlot(x_values, listOfYArrays, plot_settings={}, listOfYUncertaintiesArrays=[], showFigure=True):
+def createSimulatedResponsesPlot(x_values, listOfYArrays, plot_settings={}, listOfYUncertaintiesArrays=[], showFigure=True, directory=''):
     exportFigure = True #This variable should be moved to an argument or something in plot_settings.
     #First put some defaults in if not already defined.
     x_values = np.array(x_values)
@@ -235,24 +248,26 @@ def createSimulatedResponsesPlot(x_values, listOfYArrays, plot_settings={}, list
         ax0.legend(plot_settings['legendLabels']) #legends must be after plots are made.
     fig0.tight_layout()
     if exportFigure==True:
-        fig0.savefig(plot_settings['figure_name'] + '.png', dpi=plot_settings['dpi'])
+        fig0.savefig(directory + plot_settings['figure_name'] + '.png', dpi=plot_settings['dpi'])
     if showFigure==False:
         plt.close(fig0)
     return fig0
 
-def makeTrisurfacePlot(xValues, yValues, zValues, exportFigure = True, figure_name="TrisurfacePlot", showFigure=True):
+def makeTrisurfacePlot(xValues, yValues, zValues, exportFigure = True, figure_name="TrisurfacePlot", showFigure=True, directory=''):
+    from mpl_toolkits.mplot3d import Axes3D #Although it does not look like this is called here, the InfoGain plots will fail without this line.
     fig1, ax1 =  plt.subplots(1)
     ax1 = plt.axes(projection ='3d')
     image = ax1.plot_trisurf(xValues,yValues, zValues)
     if exportFigure == True:
-        fig1.savefig(figure_name + '.png')
+        fig1.savefig(directory + figure_name + '.png')
     if showFigure==False:
         plt.close(fig1)
     return fig1, ax1, image
 
-def makeMeshGridSurfacePlot(XX, YY, ZZ,  plot_settings = {}, exportFigure = True, figure_name="MeshGridSurfacePlot", showFigure=True):
+def makeMeshGridSurfacePlot(XX, YY, ZZ,  plot_settings = {}, exportFigure = True, figure_name="MeshGridSurfacePlot", showFigure=True, directory=''):
     #TODO: plot_settings should be used for axis labels etc, like above.
     #TODO: create a UserInput variable named info_gain_plot_settings (like what the other cases have).
+    from mpl_toolkits.mplot3d import Axes3D #I am not sure if this line is needed here, it might b.
     fig1,ax1 = plt.subplots(figsize=(5,5))
     #ax = fig.add_subplot(111, projection='3d')
     surf = ax1.pcolor(XX,YY,ZZ,cmap=matplotlib.cm.coolwarm)
@@ -264,8 +279,34 @@ def makeMeshGridSurfacePlot(XX, YY, ZZ,  plot_settings = {}, exportFigure = True
     ax1.set_title('Information Gain Surface')
     fig1.colorbar(surf, shrink=0.5, aspect=5)
     if exportFigure==True:
-        fig1.savefig(figure_name + '.png')
+        fig1.savefig(directory + figure_name + '.png')
     if showFigure==False:
         plt.close(fig1)
     return fig1, ax1#, image
-    
\ No newline at end of file
+    
+
+def density_scatter( x , y, ax = None, sort = True, bins = 20, **kwargs )   :
+    """
+    Scatter plot colored by 2d histogram
+    Sampled from Guillaume post on https://stackoverflow.com/questions/20105364/how-can-i-make-a-scatter-plot-colored-by-density-in-matplotlib/53865762#53865762
+    """
+    from matplotlib import cm
+    from matplotlib.colors import Normalize 
+    from scipy.interpolate import interpn
+    if ax is None:
+        fig , ax = plt.subplots()
+    data , x_e, y_e = np.histogram2d( x, y, bins = bins, density = True )
+    z = interpn( ( 0.5*(x_e[1:] + x_e[:-1]) , 0.5*(y_e[1:]+y_e[:-1]) ) , data , np.vstack([x,y]).T , method = "splinef2d", bounds_error = False)
+    #To be sure to plot all data
+    z[np.where(np.isnan(z))] = 0.0
+
+    # Sort the points by density, so that the densest points are plotted last
+    if sort:
+        idx = z.argsort()
+        x, y, z = x[idx], y[idx], z[idx]
+
+    ax.scatter( x, y, c=z, **kwargs )
+    norm = Normalize(vmin = np.min(z), vmax = np.max(z))
+    cbar = fig.colorbar(cm.ScalarMappable(norm = norm), ax=ax)
+    cbar.ax.set_ylabel('Density')
+    return fig, ax                                                                             
\ No newline at end of file
